@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2000-2005 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
  *
- * This file may be distributed under the terms of the Q Public License
- * as defined by Trolltech AS of Norway and appearing in the file
- * LICENSE.QPL included in the packaging of this file.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #ifndef	_TYPES_H_
 #define	_TYPES_H_
@@ -40,6 +47,8 @@ enum {
 	STATE_RUNNING = 2,
 	STATE_STOPPED = 3,
 	STATE_STOPPING = 4,
+	STATE_RESTORING = 5,
+	STATE_CHECKPOINTING = 6,
 };
 
 /* Parse errro codes */
@@ -48,6 +57,7 @@ enum {
 #define ERR_UNK		-3
 #define ERR_NOMEM	-4
 #define ERR_OTHER	-5
+#define ERR_INVAL_SKIP	-6
 
 /** VPS handler.
  */
@@ -62,5 +72,7 @@ typedef enum {
 	SKIP_ACTION_SCRIPT =	(1<<2)
 } skipFlags;
 
+typedef int (* execFn)(void *data);
+
 #endif
 
