@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2000-2005 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
  *
- * This file may be distributed under the terms of the Q Public License
- * as defined by Trolltech AS of Norway and appearing in the file
- * LICENSE.QPL included in the packaging of this file.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #ifndef _VZCTL_PARAM_H_
 #define _VZCTL_PARAM_H_
@@ -92,5 +99,16 @@
 #define	PARAM_DEF_OSTEMPLATE	328
 #define	PARAM_LOCKDIR		329
 
+#define PARAM_DUMPFILE		330
+#define PARAM_CPTCONTEXT	331
+#define PARAM_CPU_FLAGS		332
+#define PARAM_KILL		333
+#define PARAM_RESUME		334
+#define PARAM_DUMP		335
+#define PARAM_SUSPEND		336
+#define PARAM_UNDUMP		337
+#define PARAM_DUMPDIR		338
+#define PARAM_SKIPARPDETECT	339
+
 #define PARAM_LINE		"e:p:f:t:i:l:k:a:b:n:x:h"
 #endif
