@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2000-2005 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
  *
- * This file may be distributed under the terms of the Q Public License
- * as defined by Trolltech AS of Norway and appearing in the file
- * LICENSE.QPL included in the packaging of this file.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include <stdlib.h>
 #include <string.h>
@@ -18,7 +25,6 @@
 #include <arpa/inet.h>
 #include <getopt.h>
 #include <sys/stat.h>
-#include <asm/timex.h>
 #include <linux/vzcalluser.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -32,46 +38,15 @@
 #include "ub.h"
 #include "vzctl.h"
 #include "res.h"
+#include "iptables.h"
 
 
 static int _page_size;
 
-static struct iptables_s {
-	char *name;
-	unsigned long id;
-} iptables[] = {
-#ifdef VZCTL_ENV_CREATE_DATA
-	{"iptable_filter", VE_IP_FILTER},
-	{"iptable_mangle", VE_IP_MANGLE},
-	{"ipt_limit", VE_IP_MATCH_LIMIT},
-	{"ipt_multiport", VE_IP_MATCH_MULTIPORT},
-	{"ipt_tos", VE_IP_MATCH_TOS},
-	{"ipt_TOS", VE_IP_TARGET_TOS},
-	{"ipt_REJECT", VE_IP_TARGET_REJECT},
-	{"ipt_TCPMSS", VE_IP_TARGET_TCPMSS},
-	{"ipt_tcpmss", VE_IP_MATCH_TCPMSS},
-	{"ipt_ttl", VE_IP_MATCH_TTL},
-	{"ipt_LOG", VE_IP_TARGET_LOG},
-	{"ipt_length", VE_IP_MATCH_LENGTH},
-	{"ip_conntrack", VE_IP_CONNTRACK},
-	{"ip_conntrack_ftp", VE_IP_CONNTRACK_FTP},
-	{"ip_conntrack_irc", VE_IP_CONNTRACK_IRC},
-	{"ipt_conntrack", VE_IP_MATCH_CONNTRACK},
-	{"ipt_state", VE_IP_MATCH_STATE},
-	{"ipt_helper", VE_IP_MATCH_HELPER},
-	{"iptable_nat", VE_IP_NAT},
-	{"ip_nat_ftp", VE_IP_NAT_FTP},
-	{"ip_nat_irc", VE_IP_NAT_IRC},
-#ifdef VE_IP_TARGET_REDIRECT
-	{"ipt_REDIRECT", VE_IP_TARGET_REDIRECT},
-#endif
-#endif /* VZCTL_ENV_CREATE_DATA */
-	{NULL, 0}
-};
-
 static vps_config config[] = {
 /*	Op	*/
 {"LOCKDIR",	NULL, PARAM_LOCKDIR},
+{"DUMPDIR",	NULL, PARAM_DUMPDIR},
 /*	Log	*/
 {"LOGGING",	NULL, PARAM_LOGGING},
 {"LOG_LEVEL",	NULL, PARAM_LOGLEVEL},
@@ -176,6 +151,7 @@ static struct option set_opt[] = {
 {"ipadd",	required_argument, NULL, PARAM_IP_ADD},
 {"ip",		required_argument, NULL, PARAM_IP_ADD},
 {"ipdel",	required_argument, NULL, PARAM_IP_DEL},
+{"skip_arpdetect",no_argument, NULL, PARAM_SKIPARPDETECT},
 {"netdev_add",	required_argument, NULL, PARAM_NETDEV_ADD},
 {"netdev_del",	required_argument, NULL, PARAM_NETDEV_DEL},
 {"hostname",	required_argument, NULL, PARAM_HOSTNAME},
@@ -364,54 +340,36 @@ int conf_store_yesno(list_head_t *conf, char *name, int val)
 }
 
 /******************** Iptables *************************/
-static int find_ipt(const char *name)
-{
-	int i;
-
-	for (i = 0; iptables[i].name != NULL; i++) 
-		if (!strcmp(name, iptables[i].name))
-			return i;
-	return -1;
-}
-
 static int parse_iptables(env_param *env, char *val)
 {
-	int id;
 	char *token;
+	struct iptables_s *ipt;
+	int ret;
 
 	if ((token = strtok(val, "\t ")) == NULL)
 		return 0;
+	ret = 0;
 	do {
-		if ((id = find_ipt(token)) < 0) { 
-			logger(0, 0, "Unknown iptable module %s", token);
-			env->ipt_mask = 0;
-			return ERR_INVAL;
+		if ((ipt = find_ipt(token)) == NULL) { 
+			logger(0, 0, "Warning: Unknown iptable module: %s,"
+				" skipped", token);
+			ret = ERR_INVAL_SKIP;
+			continue;
 		}
-		env->ipt_mask |= iptables[id].id;
+		env->ipt_mask |= ipt->id;
 	} while ((token = strtok(NULL, "\t ")));
-	return 0;
+	return ret;
 }
 
 static void store_iptables(unsigned long ipt_mask, vps_config *conf,
 	list_head_t *conf_h)
 {
-	int i, r;
 	char buf[STR_SIZE];
-	char *sp, *ep;
+	int r;
 
-	sp = buf;
-	ep = buf + sizeof(buf);
 	r = snprintf(buf, sizeof(buf), "%s=\"", conf->name);
-	sp += r;
-	for (i = 0; iptables[i].name != NULL; i++) {
-		if (!VE_IPT_CMP(ipt_mask, iptables[i].id))
-			continue;
-		r = snprintf(sp, ep - sp, "%s ", iptables[i].name);
-		if (r < 0 || sp + r >= ep)
-			break;
-		sp += r;
-	}
-	snprintf(sp, ep - sp, "\"");
+	ipt_mask2str(ipt_mask, buf + r, sizeof(buf) - r - 1);
+	strcat(buf, "\"");
 	add_str_param(conf_h, buf);
 }
 
@@ -1188,6 +1146,9 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_LOCKDIR:
 		ret = conf_parse_str(&vps_p->opt.lockdir, val, 1);
 		break;
+	case PARAM_DUMPDIR:
+		ret = conf_parse_str(&vps_p->res.cpt.dumpdir, val, 1);
+		break;
 	case PARAM_LOGGING:
 		ret = conf_parse_yesno(&vps_p->log.enable, val, 1);
 		break;
@@ -1233,6 +1194,9 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_IP_DEL:
 		ret = parse_ip(vps_p, val, id);
 		break;
+	case PARAM_SKIPARPDETECT:
+		vps_p->res.net.skip_arpdetect = YES;
+		break;
 	case PARAM_NETDEV_ADD:
 		ret = add_netdev(&vps_p->res.net, val);
 		break;
@@ -1408,6 +1372,8 @@ int vps_parse_config(envid_t veid, char *path, vps_param *vps_p,
 			continue;
 		if (!ret) {
 			continue;
+		} else if (ret == ERR_INVAL_SKIP) {
+			continue;
 		} else if (ret == ERR_DUP) {
 			logger(0, 0, "Warning: dup for %s=%s in line %d"
 				" is ignored", ltoken, rtoken, line);
@@ -1687,6 +1653,12 @@ static void free_dq(dq_param *dq)
 	FREE_P(dq->exptime)
 	FREE_P(dq->ugidlimit)
 }
+
+static void free_cpt(cpt_param *cpt)
+{
+	FREE_P(cpt->dumpdir)
+	FREE_P(cpt->dumpfile)
+}
 #undef FREE_P
 
 static void free_vps_res(vps_res *res)
@@ -1699,6 +1671,7 @@ static void free_vps_res(vps_res *res)
 	free_dev(&res->dev);
 	free_cpu(&res->cpu);
 	free_dq(&res->dq);
+	free_cpt(&res->cpt);
 }
 
 void free_vps_param(vps_param *param)
@@ -1789,6 +1762,8 @@ static void merge_net(net_param *dst, net_param *src)
 {
 	MERGE_LIST(ip)
 	MERGE_LIST(dev)
+	MERGE_INT(delall)
+	MERGE_INT(skip_arpdetect)
 }
 
 static void merge_misc(misc_param *dst, misc_param *src)
@@ -1833,6 +1808,15 @@ static void merge_env(env_param *dst, env_param *src)
 	MERGE_INT(ipt_mask);
 }
 
+static void merge_cpt(cpt_param *dst, cpt_param *src)
+{
+	MERGE_STR(dumpdir);
+	MERGE_STR(dumpfile);
+	MERGE_INT(ctx);
+	MERGE_INT(cpu_flags);
+	MERGE_INT(cmd);
+}
+
 static int merge_res(vps_res *dst, vps_res *src)
 {
 	merge_fs(&dst->fs, &src->fs);
@@ -1845,8 +1829,8 @@ static int merge_res(vps_res *dst, vps_res *src)
 	merge_dq(&dst->dq, &src->dq);
 	merge_dev(&dst->dev, &src->dev);
 	merge_env(&dst->env, &src->env);
+	merge_cpt(&dst->cpt, &src->cpt);
 	return 0;
-
 }
 
 int merge_vps_param(vps_param *dst, vps_param *src)
