@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2000-2005 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
  *
- * This file may be distributed under the terms of the Q Public License
- * as defined by Trolltech AS of Norway and appearing in the file
- * LICENSE.QPL included in the packaging of this file.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include <stdlib.h>
@@ -15,7 +22,6 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
-#include <ub/beancounter.h>
 
 #include "types.h"
 #include "ub.h"
@@ -23,6 +29,15 @@
 #include "vzerror.h"
 #include "logger.h"
 
+#ifndef __NR_setublimit
+#ifdef __ia64__
+#define __NR_setublimit	1507
+#elif __x86_64__
+#define __NR_setublimit	502
+#else
+#define __NR_setublimit	512
+#endif
+#endif
 
 static inline int setublimit(uid_t uid, unsigned long resource,
         unsigned long *rlim)
