@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2000-2005 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
  *
- * This file may be distributed under the terms of the Q Public License
- * as defined by Trolltech AS of Norway and appearing in the file
- * LICENSE.QPL included in the packaging of this file.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include <stdlib.h>
@@ -15,7 +22,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <getopt.h>
-#include <asm/timex.h>
 #include <linux/vzcalluser.h>
 #include <errno.h>
 #include <signal.h>
@@ -51,7 +57,7 @@ int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 		ret = vps_parse_opt(veid, param, c, optarg, &g_action);
 		if (!ret)
 			continue;
-		else if (ret == ERR_INVAL) {
+		else if (ret == ERR_INVAL || ret == ERR_INVAL_SKIP) {
 			if (option_index < 0) {
 				logger(0, 0, "Bad parameter for"
 					" -%c : %s", c, optarg);
@@ -63,14 +69,12 @@ int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 				return VZ_INVALID_PARAMETER_VALUE;
 			}
 		} else if (ret == ERR_UNK) {
-			if (option_index < 0) {
+			if (option_index < 0)
 				logger(0, 0, "Invalid option -%c", c);
-				return VZ_INVALID_PARAMETER_SYNTAX;
-			} else {
+			else
 				logger(0, 0, "Invalid option --%s",
 					opt[option_index].name);
-				return VZ_INVALID_PARAMETER_SYNTAX;
-			}
+			return VZ_INVALID_PARAMETER_SYNTAX;
                 }
 	}
 	if (optind < argc) {
@@ -227,6 +231,151 @@ static int destroy(vps_handler *h, envid_t veid, vps_param *g_p,
 	return vps_destroy(h, veid, &g_p->res.fs);
 }
 
+static int parse_chkpnt_opt(int argc, char **argv, vps_param *vps_p)
+{
+	int c, ret;
+	int option_index;
+	cpt_param *cpt = &vps_p->res.cpt;
+	static struct option chkpnt_options[] = {
+	/*	sub commands	*/
+	{"dump",	no_argument, NULL, PARAM_DUMP},
+	{"suspend",	no_argument, NULL, PARAM_SUSPEND},
+	{"resume",      no_argument, NULL, PARAM_RESUME},
+	{"kill",        no_argument, NULL, PARAM_KILL},
+	{"skip_arpdetect", no_argument, NULL, PARAM_SKIPARPDETECT},
+	/*	flags		*/
+	{"flags",	required_argument, NULL, PARAM_CPU_FLAGS},
+	{"context",	required_argument, NULL, PARAM_CPTCONTEXT},
+	{"dumpfile",	required_argument, NULL, PARAM_DUMPFILE},
+	{ NULL, 0, NULL, 0 }
+	};
+
+	ret = 0;
+	while (1) {
+		option_index = -1;
+		c = getopt_long (argc, argv, "", chkpnt_options, NULL); 
+		if (c == -1)
+			break;
+		switch (c) {
+		case PARAM_DUMPFILE:
+			cpt->dumpfile = strdup(optarg);
+			break;
+		case PARAM_CPTCONTEXT:
+			cpt->ctx = strtoul(optarg, NULL, 16);
+			break;
+		case PARAM_CPU_FLAGS:
+			cpt->cpu_flags = strtoul(optarg, NULL, 0);
+			break;
+		case PARAM_DUMP:
+			if (cpt->cmd)
+				goto err_syntax;
+			cpt->cmd = CMD_DUMP;
+			break;
+		case PARAM_KILL:
+			if (cpt->cmd)
+				goto err_syntax;
+			cpt->cmd = CMD_KILL;
+			break;
+		case PARAM_RESUME:
+			if (cpt->cmd)
+				goto err_syntax;
+			cpt->cmd = CMD_RESUME;
+			break;
+		case PARAM_SUSPEND:
+			if (cpt->cmd)
+				goto err_syntax;
+			cpt->cmd = CMD_SUSPEND;
+			break;
+		case PARAM_SKIPARPDETECT:
+			vps_p->res.net.skip_arpdetect = YES;
+			break;
+		default:
+			if (option_index < 0)
+				logger(0, 0, "Invalid option -%c", c);
+			else
+				logger(0, 0, "Invalid option --%s", 
+					chkpnt_options[option_index].name);
+			return VZ_INVALID_PARAMETER_SYNTAX;
+		}
+	}
+	/* Do full checkpointing */
+	if (!cpt->cmd)
+		cpt->cmd = CMD_CHKPNT;
+	return 0;
+
+err_syntax:
+	logger(0, 0, "Invalid syntax: only one sub command may be used");
+	return VZ_INVALID_PARAMETER_SYNTAX;
+}
+
+static int parse_restore_opt(int argc, char **argv, vps_param *vps_p)
+{
+	int c;
+	int option_index;
+	cpt_param *cpt = &vps_p->res.cpt;
+	static struct option restore_options[] = {
+	/*	sub commands	*/
+	{"undump",	no_argument, NULL, PARAM_UNDUMP},
+	{"kill",        no_argument, NULL, PARAM_KILL},
+	{"resume",      no_argument, NULL, PARAM_RESUME},
+	/*	flags		*/
+	{"dumpfile",	required_argument, NULL, PARAM_DUMPFILE},
+	{"flags",	required_argument, NULL, PARAM_CPU_FLAGS},
+	{"context",	required_argument, NULL, PARAM_CPTCONTEXT},
+	{"skip_arpdetect", no_argument, NULL, PARAM_SKIPARPDETECT},
+	{ NULL, 0, NULL, 0 }
+	};
+
+	while (1) {
+		option_index = -1;
+		c = getopt_long (argc, argv, "", restore_options, NULL); 
+		if (c == -1)
+			break;
+		switch (c) {
+		case PARAM_DUMPFILE:
+			cpt->dumpfile = strdup(optarg);
+			break;
+		case PARAM_CPTCONTEXT:
+			cpt->ctx = strtoul(optarg, NULL, 16);
+			break;
+		case PARAM_CPU_FLAGS:
+			cpt->cpu_flags = strtoul(optarg, NULL, 0);
+			break;
+		case PARAM_UNDUMP:
+			if (cpt->cmd)
+				goto err_syntax;
+			cpt->cmd = CMD_UNDUMP;
+			break;
+		case PARAM_KILL:
+			if (cpt->cmd)
+				goto err_syntax;
+			cpt->cmd = CMD_KILL;
+			break;
+		case PARAM_RESUME:
+			if (cpt->cmd)
+				goto err_syntax;
+			cpt->cmd = CMD_RESUME;
+			break;
+		case PARAM_SKIPARPDETECT:
+			vps_p->res.net.skip_arpdetect = YES;
+			break;
+		default:
+			if (option_index < 0)
+				logger(0, 0, "Invalid option -%c", c);
+			else
+				logger(0, 0, "Invalid option --%s", 
+					restore_options[option_index].name);
+			return VZ_INVALID_PARAMETER_SYNTAX;
+		}
+	}
+	/* Do full restore */
+	if (!cpt->cmd)
+		cpt->cmd = CMD_RESTORE;
+	return 0;
+err_syntax:
+	logger(0, 0, "Invalid syntax: only one sub command may be used");
+	return VZ_INVALID_PARAMETER_SYNTAX;
+}
 int check_set_mode(vps_handler *h, envid_t veid, int setmode, int apply,
 	vps_res *new_res, vps_res *old_res)
 {
@@ -470,6 +619,29 @@ static int exec(vps_handler *h, int action, envid_t veid, char *root, int argc,
 	return ret;
 }
 
+static int chkpnt(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *cmd_p)
+{
+	int cmd;
+
+	cmd = cmd_p->res.cpt.cmd;
+	merge_vps_param(g_p, cmd_p);
+	if (cmd == CMD_KILL || cmd == CMD_RESUME) 
+		return cpt_cmd(h, veid, CMD_CHKPNT, &cmd_p->res.cpt, g_p);
+	return vps_chkpnt(h, veid, g_p, cmd, &cmd_p->res.cpt);
+}
+
+static int restore(vps_handler *h, envid_t veid, vps_param *g_p,
+	vps_param *cmd_p)
+{
+	int cmd;
+
+	cmd = cmd_p->res.cpt.cmd;
+	merge_vps_param(g_p, cmd_p);
+	if (cmd == CMD_KILL || cmd == CMD_RESUME)
+		return cpt_cmd(h, veid, CMD_RESTORE, &cmd_p->res.cpt, g_p);
+	return vps_restore(h, veid, g_p, cmd, &cmd_p->res.cpt);
+}
+
 static int show_status(vps_handler *h, envid_t veid, vps_param *param)
 {
 	int exist = 0, mounted = 0, run = 0;
@@ -542,6 +714,12 @@ int parse_action_opt(envid_t veid, int action, int argc, char *argv[],
 	case ACTION_CUSTOM:
 		ret = parse_custom_opt(veid, argc, argv, param, name);
 		break;
+	case ACTION_CHKPNT:
+		ret = parse_chkpnt_opt(argc, argv, param);
+		break;
+	case ACTION_RESTORE:
+		ret = parse_restore_opt(argc, argv, param);
+		break;
 	default :
 		if ((argc - 1) > 0) {
    	                fprintf (stderr, "Invalid options: ");
@@ -643,9 +821,24 @@ int run_action(envid_t veid, int action, vps_param *g_p, vps_param *vps_p,
 		if (ret && action == ACTION_EXEC)
 			ret = VZ_COMMAND_EXECUTION_ERROR;
 		break;
+	case ACTION_CHKPNT:
+		ret = chkpnt(h, veid, g_p, cmd_p);
+		break;
+	case ACTION_RESTORE:
+		ret = restore(h, veid, g_p, cmd_p);
+		break;
 	case ACTION_RUNSCRIPT:
 		ret = vps_run_script(h, veid, argv[0], g_p);
 		break;
+	case ACTION_QUOTAON:
+		ret = vps_quotaon(veid, g_p->res.fs.private, &g_p->res.dq);
+		break;
+	case ACTION_QUOTAOFF:
+		ret = vps_quotaoff(veid, &g_p->res.dq);
+		break;
+	case ACTION_QUOTAINIT:
+		ret = quota_init(veid, g_p->res.fs.private, &g_p->res.dq);
+		break;
 	case ACTION_CUSTOM:
 		ret = mod_setup(h, veid, 0, 0, &g_action, g_p);
 		break;
