@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2000-2005 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2006 SWsoft. All rights reserved.
  *
- * This file may be distributed under the terms of the Q Public License
- * as defined by Trolltech AS of Norway and appearing in the file
- * LICENSE.QPL included in the packaging of this file.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include <stdlib.h>
@@ -56,6 +63,8 @@ void usage()
 "vzctl start <veid> [--force]\n"
 "vzctl exec | exec2 <veid> <command> [arg ...]\n"
 "vzctl runscript <veid> <script>\n"
+"vzctl chkpnt <vpsid> [--dumpfile <name>]\n"
+"vzctl restore <vpsid> [--dumpfile <name>]\n"
 "vzctl set veid [--save] [--setmode restart|ignore]\n"
 "   [--ipadd <addr>] [--ipdel <addr>|all] [--hostname <name>]\n"
 "   [--nameserver <addr>] [--searchdomain <name>] [--onboot yes|no]\n"
@@ -164,6 +173,16 @@ int main(int argc, char *argv[], char *envp[])
 	} else if (!strcmp(argv[1], "status")) {
 		action = ACTION_STATUS;
 		quiet = 1;
+	} else if (!strcmp(argv[1], "chkpnt")) {
+		action = ACTION_CHKPNT;
+	} else if (!strcmp(argv[1], "restore")) {
+		action = ACTION_RESTORE;
+	} else if (!strcmp(argv[1], "quotaon")) {
+		action = ACTION_QUOTAON;
+	} else if (!strcmp(argv[1], "quotaoff")) {
+		action = ACTION_QUOTAOFF;
+	} else if (!strcmp(argv[1], "quotainit")) {
+		action = ACTION_QUOTAINIT;
 	} else if (!strcmp(argv[1], "--help")) {
 		usage();
 		exit(0);
